@@ -17,7 +17,8 @@
 //! // These closures should return a `Reader` instance (e.g. `EndianSlice`).
 //! let loader = |section: gimli::SectionId| { get_file_section_reader(section.name()) };
 //! let sup_loader = |section: gimli::SectionId| { get_sup_file_section_reader(section.name()) };
-//! let dwarf = gimli::Dwarf::load(loader, sup_loader)?;
+//! let mut dwarf = gimli::Dwarf::load(loader)?;
+//! dwarf.load_sup(sup_loader)?;
 //!
 //! // Iterate over all compilation units.
 //! let mut iter = dwarf.units();
@@ -47,14 +48,14 @@
 //!
 //!   * [An `addr2line` clone](https://github.com/gimli-rs/addr2line)
 //!
-//!   * [`ddbug`](https://github.com/philipc/ddbug), a utility giving insight into
+//!   * [`ddbug`](https://github.com/gimli-rs/ddbug), a utility giving insight into
 //!     code generation by making debugging information readable
 //!
 //!   * [`dwprod`](https://github.com/fitzgen/dwprod), a tiny utility to list the
 //!     compilers used to create each compilation unit within a shared library or
 //!     executable (via `DW_AT_producer`)
 //!
-//!   * [`dwarf-validate`](http://github.com/gimli-rs/gimli/blob/master/examples/dwarf-validate.rs),
+//!   * [`dwarf-validate`](https://github.com/gimli-rs/gimli/blob/master/examples/dwarf-validate.rs),
 //!     a program to validate the integrity of some DWARF and its references
 //!     between sections and compilation units.
 //!
@@ -104,6 +105,10 @@
 //!
 //!   * [`DebugTypes`](./struct.DebugTypes.html): The `.debug_types` section.
 //!
+//!   * [`DebugCuIndex`](./struct.DebugCuIndex.html): The `.debug_cu_index` section.
+//!
+//!   * [`DebugTuIndex`](./struct.DebugTuIndex.html): The `.debug_tu_index` section.
+//!
 //!   * [`EhFrame`](./struct.EhFrame.html): The `.eh_frame` section.
 //!
 //!   * [`EhFrameHdr`](./struct.EhFrameHdr.html): The `.eh_frame_hdr` section.
@@ -142,6 +147,8 @@
 //! into your code:
 //!
 //! ```
+//! # #[cfg(feature = "fallible-iterator")]
+//! # fn foo() {
 //! // Use the `FallibleIterator` trait so its methods are in scope!
 //! use fallible_iterator::FallibleIterator;
 //! use gimli::{DebugAranges, EndianSlice, LittleEndian};
@@ -149,64 +156,90 @@
 //! fn find_sum_of_address_range_lengths(aranges: DebugAranges<EndianSlice<LittleEndian>>)
 //!     -> gimli::Result<u64>
 //! {
-//!     // `DebugAranges::items` returns a `FallibleIterator`!
-//!     aranges.items()
+//!     // `DebugAranges::headers` returns a `FallibleIterator`!
+//!     aranges.headers()
+//!         // `flat_map` is provided by `FallibleIterator`!
+//!         .flat_map(|header| Ok(header.entries()))
 //!         // `map` is provided by `FallibleIterator`!
 //!         .map(|arange| Ok(arange.length()))
 //!         // `fold` is provided by `FallibleIterator`!
 //!         .fold(0, |sum, len| Ok(sum + len))
 //! }
-//!
+//! # }
 //! # fn main() {}
 //! ```
 
-use std::fmt::{self, Debug};
-use std::result;
+use core::fmt::{self, Debug};
+use core::result;
 #[cfg(feature = "std")]
 use std::{error, io};
 
 use crate::common::{Register, SectionId};
 use crate::constants;
 
+mod util;
+pub use util::*;
+
 mod addr;
 pub use self::addr::*;
 
 mod cfi;
 pub use self::cfi::*;
 
+#[cfg(feature = "read")]
 mod dwarf;
+#[cfg(feature = "read")]
 pub use self::dwarf::*;
 
 mod endian_slice;
 pub use self::endian_slice::*;
 
+#[cfg(feature = "endian-reader")]
 mod endian_reader;
+#[cfg(feature = "endian-reader")]
 pub use self::endian_reader::*;
 
 mod reader;
 pub use self::reader::*;
 
+#[cfg(feature = "read")]
 mod abbrev;
+#[cfg(feature = "read")]
 pub use self::abbrev::*;
 
 mod aranges;
 pub use self::aranges::*;
 
+mod index;
+pub use self::index::*;
+
+#[cfg(feature = "read")]
+mod lazy;
+
+#[cfg(feature = "read")]
 mod line;
+#[cfg(feature = "read")]
 pub use self::line::*;
 
+mod lists;
+
 mod loclists;
 pub use self::loclists::*;
 
+#[cfg(feature = "read")]
 mod lookup;
 
 mod op;
 pub use self::op::*;
 
+#[cfg(feature = "read")]
 mod pubnames;
+#[cfg(feature = "read")]
 pub use self::pubnames::*;
 
+#[cfg(feature = "read")]
 mod pubtypes;
+#[cfg(feature = "read")]
 pub use self::pubtypes::*;
 
 mod rnglists;
@@ -215,12 +248,22 @@ pub use self::rnglists::*;
 mod str;
 pub use self::str::*;
 
+/// An offset into the current compilation or type unit.
+#[derive(Debug, Clone, Copy, PartialEq, Eq, Ord, PartialOrd, Hash)]
+pub struct UnitOffset<T = usize>(pub T);
+
+#[cfg(feature = "read")]
 mod unit;
+#[cfg(feature = "read")]
 pub use self::unit::*;
 
 mod value;
 pub use self::value::*;
 
+/// Indicates that storage should be allocated on heap.
+#[derive(Debug, Clone, Copy, PartialEq, Eq)]
+pub struct StoreOnHeap;
+
 /// `EndianBuf` has been renamed to `EndianSlice`. For ease of upgrading across
 /// `gimli` versions, we export this type alias.
 #[deprecated(note = "EndianBuf has been renamed to EndianSlice, use that instead.")]
@@ -312,6 +355,8 @@ pub enum Error {
     /// An unrecognized operation was found while parsing a DWARF
     /// expression.
     InvalidExpression(constants::DwOp),
+    /// An unsupported operation was found while evaluating a DWARF expression.
+    UnsupportedEvaluation,
     /// The expression had a piece followed by an expression
     /// terminator without a piece.
     InvalidPiece,
@@ -359,9 +404,9 @@ pub enum Error {
     UnsupportedRegister(u64),
     /// The CFI program defined more register rules than we have storage for.
     TooManyRegisterRules,
-    /// Attempted to push onto the CFI stack, but it was already at full
-    /// capacity.
-    CfiStackFull,
+    /// Attempted to push onto the CFI or evaluation stack, but it was already
+    /// at full capacity.
+    StackFull,
     /// The `.eh_frame_hdr` binary search table claims to be variable-length encoded,
     /// which makes binary search impossible.
     VariableLengthSearchTable,
@@ -379,12 +424,22 @@ pub enum Error {
     MissingFileEntryFormatPath,
     /// Expected an attribute value to be a string form.
     ExpectedStringAttributeValue,
+    /// `DW_FORM_implicit_const` used in an invalid context.
+    InvalidImplicitConst,
+    /// Invalid section count in `.dwp` index.
+    InvalidIndexSectionCount,
+    /// Invalid slot count in `.dwp` index.
+    InvalidIndexSlotCount,
+    /// Invalid hash row in `.dwp` index.
+    InvalidIndexRow,
+    /// Unknown section type in `.dwp` index.
+    UnknownIndexSection,
 }
 
 impl fmt::Display for Error {
     #[inline]
-    fn fmt(&self, f: &mut fmt::Formatter) -> ::std::result::Result<(), fmt::Error> {
-        Debug::fmt(self, f)
+    fn fmt(&self, f: &mut fmt::Formatter) -> ::core::result::Result<(), fmt::Error> {
+        write!(f, "{}", self.description())
     }
 }
 
@@ -460,6 +515,7 @@ impl Error {
             Error::NotEnoughStackItems => "Not enough items on stack when evaluating expression",
             Error::TooManyIterations => "Too many iterations to evaluate DWARF expression",
             Error::InvalidExpression(_) => "Invalid opcode in DWARF expression",
+            Error::UnsupportedEvaluation => "Unsupported operation when evaluating expression",
             Error::InvalidPiece => {
                 "DWARF expression has piece followed by non-piece expression at end"
             }
@@ -504,7 +560,7 @@ impl Error {
             Error::TooManyRegisterRules => {
                 "The CFI program defined more register rules than we have storage for."
             }
-            Error::CfiStackFull => {
+            Error::StackFull => {
                 "Attempted to push onto the CFI stack, but it was already at full capacity."
             }
             Error::VariableLengthSearchTable => {
@@ -522,16 +578,17 @@ impl Error {
             Error::ExpectedStringAttributeValue => {
                 "Expected an attribute value to be a string form."
             }
+            Error::InvalidImplicitConst => "DW_FORM_implicit_const used in an invalid context.",
+            Error::InvalidIndexSectionCount => "Invalid section count in `.dwp` index.",
+            Error::InvalidIndexSlotCount => "Invalid slot count in `.dwp` index.",
+            Error::InvalidIndexRow => "Invalid hash row in `.dwp` index.",
+            Error::UnknownIndexSection => "Unknown section type in `.dwp` index.",
         }
     }
 }
 
 #[cfg(feature = "std")]
-impl error::Error for Error {
-    fn description(&self) -> &str {
-        Error::description(self)
-    }
-}
+impl error::Error for Error {}
 
 #[cfg(feature = "std")]
 impl From<io::Error> for Error {
@@ -564,10 +621,22 @@ pub trait Section<R>: From<R> {
         Self::id().name()
     }
 
+    /// Returns the ELF section name (if any) for this type when used in a dwo
+    /// file.
+    fn dwo_section_name() -> Option<&'static str> {
+        Self::id().dwo_name()
+    }
+
+    /// Returns the XCOFF section name (if any) for this type when used in a XCOFF
+    /// file.
+    fn xcoff_section_name() -> Option<&'static str> {
+        Self::id().xcoff_name()
+    }
+
     /// Try to load the section using the given loader function.
-    fn load<F, E>(f: F) -> std::result::Result<Self, E>
+    fn load<F, E>(f: F) -> core::result::Result<Self, E>
     where
-        F: FnOnce(SectionId) -> std::result::Result<R, E>,
+        F: FnOnce(SectionId) -> core::result::Result<R, E>,
     {
         f(Self::id()).map(From::from)
     }
@@ -577,6 +646,18 @@ pub trait Section<R>: From<R> {
     where
         R: Reader;
 
+    /// Returns the subrange of the section that is the contribution of
+    /// a unit in a `.dwp` file.
+    fn dwp_range(&self, offset: u32, size: u32) -> Result<Self>
+    where
+        R: Reader,
+    {
+        let mut data = self.reader().clone();
+        data.skip(R::Offset::from_u32(offset))?;
+        data.truncate(R::Offset::from_u32(size))?;
+        Ok(data.into())
+    }
+
     /// Returns the `Reader` for this section.
     fn lookup_offset_id(&self, id: ReaderOffsetId) -> Option<(SectionId, R::Offset)>
     where
